@@ -1,18 +1,19 @@
 import logging
 
-from dvc.exceptions import ReproductionError, InvalidArgumentError
+from dvc.exceptions import InvalidArgumentError, ReproductionError
+from dvc.repo.experiments import UnchangedExperimentError
 from dvc.repo.scm_context import scm_context
+
 from . import locked
 from .graph import get_pipeline, get_pipelines
 
-
 logger = logging.getLogger(__name__)
 
 
 def _reproduce_stage(stage, **kwargs):
-    if stage.locked:
+    if stage.frozen and not stage.is_import:
         logger.warning(
-            "{} is locked. Its dependencies are"
+            "{} is frozen. Its dependencies are"
             " not going to be reproduced.".format(stage)
         )
 
@@ -34,7 +35,7 @@ def _get_active_graph(G):
 
     active = G.copy()
     for stage in G:
-        if not stage.locked:
+        if not stage.frozen:
             continue
         active.remove_edges_from(G.out_edges(stage))
         for edge in G.out_edges(stage):
@@ -61,14 +62,27 @@ def reproduce(
     all_pipelines=False,
     **kwargs
 ):
-    from ..dvcfile import Dvcfile
     from dvc.utils import parse_target
 
+    assert target is None or isinstance(target, str)
     if not target and not all_pipelines:
         raise InvalidArgumentError(
             "Neither `target` nor `--all-pipelines` are specified."
         )
 
+    experiment = kwargs.pop("experiment", False)
+    if experiment and self.experiments:
+        try:
+            return self.experiments.new(
+                target=target,
+                recursive=recursive,
+                all_pipelines=all_pipelines,
+                **kwargs
+            )
+        except UnchangedExperimentError:
+            # If experiment contains no changes, just run regular repro
+            pass
+
     interactive = kwargs.get("interactive", False)
     if not interactive:
         kwargs["interactive"] = self.config["core"].get("interactive", False)
@@ -81,8 +95,7 @@ def reproduce(
         if all_pipelines:
             pipelines = active_pipelines
         else:
-            dvcfile = Dvcfile(self, path)
-            stage = dvcfile.stages[name]
+            stage = self.get_stage(path, name)
             pipelines = [get_pipeline(active_pipelines, stage)]
 
         targets = []
@@ -97,12 +110,7 @@ def reproduce(
 
 
 def _reproduce_stages(
-    G,
-    stages,
-    downstream=False,
-    ignore_build_cache=False,
-    single_item=False,
-    **kwargs
+    G, stages, downstream=False, single_item=False, **kwargs
 ):
     r"""Derive the evaluation of the given node for the given graph.
 
@@ -167,12 +175,18 @@ def _reproduce_stages(
         if stage not in pipeline:
             pipeline.append(stage)
 
+    force_downstream = kwargs.pop("force_downstream", False)
     result = []
+    # `ret` is used to add a cosmetic newline.
+    ret = []
     for stage in pipeline:
+        if ret:
+            logger.info("")
+
         try:
             ret = _reproduce_stage(stage, **kwargs)
 
-            if len(ret) != 0 and ignore_build_cache:
+            if len(ret) != 0 and force_downstream:
                 # NOTE: we are walking our pipeline from the top to the
                 # bottom. If one stage is changed, it will be reproduced,
                 # which tells us that we should force reproducing all of
